#include "StdAfx.h"

#include "Globals.h"
#include <stdio.h>

void FullHelp()
{
_tprintf(_T("Send - Sends keystrokes and windows messages to applications\n"));
_tprintf(_T("\n"));
_tprintf(_T("======================================================================\n"));
_tprintf(_T("\n"));
_tprintf(_T("Usage:  send [-rcflwdhna] [-n NewName] [-s \"Class\"] \"application\" \"keys\" [time]\n"));
_tprintf(_T("\n"));
_tprintf(_T("\"application\" is a case-insensitive string that matches part of a window\n"));
_tprintf(_T("caption.  For example, if you are running notepad, its title bar will be\n"));
_tprintf(_T("something similar to \"MyFile.txt - Notepad\".  You could have send.exe type\n"));
_tprintf(_T("the text \"Hello\" by using the following command:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send notepad Hello\n"));
_tprintf(_T("\n"));
_tprintf(_T("Spaces in the key string need to be quoted, as in:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send notepad \"Hello everyone!\"\n"));
_tprintf(_T("    \n"));
_tprintf(_T("[time] is a string representation of a time at which you want the keys to\n"));
_tprintf(_T("be sent to the application.  The application window need not exist until\n"));
_tprintf(_T("that time.  For example, to send the F6 key to VirtualDub at 4:55 PM, use\n"));
_tprintf(_T("the following syntax:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send virtualdub \\{F6} 4:55pm\n"));
_tprintf(_T("\n"));
_tprintf(_T("The time string accepts several different time formats (it uses the Windows\n"));
_tprintf(_T("function VarDateFromStr), including dates.  Dates with spaces must be\n"));
_tprintf(_T("quoted, as in:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send virtualdub \\{F6} \"April 1, 2001 10:55am\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("Options:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -r  Uses a regular expression for the search string.  For example,\n"));
_tprintf(_T("        \"^notepad$\" would match only a window titled exactly \"Notepad\" and\n"));
_tprintf(_T("        not one name \"Untitled - Notepad\".  A regular expression of \".\"\n"));
_tprintf(_T("        will match all windows that have a title of at least one character.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -c  Uses a case-sensitive search, so that \"notepad\" will not match\n"));
_tprintf(_T("        \"Notepad\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -l  Lists all of the window captions that \"send\" finds before\n"));
_tprintf(_T("        matching the search string.  Useful for discoving the precise\n"));
_tprintf(_T("        titles of windows.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -f  Sends the specified string to the window repeatedly.  This is most\n"));
_tprintf(_T("        useful when the keystrokes you are sending will cause the window to\n"));
_tprintf(_T("        close, and you want to send the same keystrokes to the window again\n"));
_tprintf(_T("        when it reappears (with the -w options).\n"));
_tprintf(_T("        \n"));
_tprintf(_T("    -g  Sends the specified string to whichever window happens to be in\n"));
_tprintf(_T("        the foreground.  Useful for arbitrary mouse movements that do not\n"));
_tprintf(_T("        need a particular window to be focused.  If you specify the \"-g\"\n"));
_tprintf(_T("        option, you must still specify a window caption, but it is ignored.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("    -b  Does not bring the specified window to the foreground.  Keystrokes\n"));
_tprintf(_T("        will still be sent to whatever window is in the foreground, but\n"));
_tprintf(_T("        you may send windows messages and position/size commands to a\n"));
_tprintf(_T("        background window (useful for moving a window without bringing it\n"));
_tprintf(_T("        to the foreground first).  Using this option implies -d.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("    -w  Waits for the window to exist before sending the keystrokes.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("    -W  Similar to -w, but waits for the window to be enabled.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -d  Disables switching back to the process that launched send.exe\n"));
_tprintf(_T("        after sending the input string.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -h  Includes hidden windows in the search.  Normally, a nonvisible\n"));
_tprintf(_T("        window is not a desired target for input operations.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -n  Gives the found window a new caption.  This can be useful when you\n"));
_tprintf(_T("        want to send an input string that will take some time to process,\n"));
_tprintf(_T("        and then use another \"send\" command to wait for the new window\n"));
_tprintf(_T("        caption to exist.\n"));
_tprintf(_T("        \n"));
_tprintf(_T("    -a  Sends the commands to every window matching the expression, not\n"));
_tprintf(_T("        just the first one.\n"));
_tprintf(_T("        \n"));
_tprintf(_T("    -s  The window must also match the class name provided.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("    -i  The window message is sent to the child window specified by\n"));
_tprintf(_T("        the given dialog ID.\n"));
_tprintf(_T("        \n"));
_tprintf(_T("    -p  Sends the actual message to the parent of the indicated window\n"));
_tprintf(_T("        (using -i) instead of the window itself.  When using messages that\n"));
_tprintf(_T("        include notifications such as BN_CLICKED, this is typically useful.\n"));
_tprintf(_T("        \n"));
_tprintf(_T("    -x  Waits for the window to no longer exist before sending the keystrokes.\n"));
_tprintf(_T("\n"));
_tprintf(_T("    -X  Similar to -x, but waits for the window to be disabled.\n"));
_tprintf(_T("        \n"));
_tprintf(_T("\n"));
_tprintf(_T("======================================================================\n"));
_tprintf(_T("\n"));
_tprintf(_T("If you want to send a special key, such as F6 or Escape, use the following\n"));
_tprintf(_T("tables (examples are for notepad or other such edit controls):\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Keyboard Modifiers:\n"));
_tprintf(_T("    \\a      Press the ALT key\n"));
_tprintf(_T("    \\A      Release the ALT key\n"));
_tprintf(_T("    \\s      Press the SHIFT key\n"));
_tprintf(_T("    \\S      Release the SHIFT key\n"));
_tprintf(_T("    \\c      Press the CONTROL key\n"));
_tprintf(_T("    \\C      Release the CONTROL key\n"));
_tprintf(_T("    \\p      Press the APPLICATIONS key\n"));
_tprintf(_T("    \\P      Release the APPLICATIONS key\n"));
_tprintf(_T("    \\w      Press the WINDOWS key\n"));
_tprintf(_T("    \\W      Release the WINDWOS key\n"));
_tprintf(_T("\n"));
_tprintf(_T("For example, \"\\shello\\S\" would send \"shift, h, e, l, l, o, unshift\" and\n"));
_tprintf(_T("create the word \"HELLO\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Arrow Keys:\n"));
_tprintf(_T("    \\u      UP arrow       \n"));
_tprintf(_T("    \\d      DOWN arrow       \n"));
_tprintf(_T("    \\l      LEFT arrow       \n"));
_tprintf(_T("    \\r      RIGHT arrow       \n"));
_tprintf(_T("\n"));
_tprintf(_T("For example, \"hllo\\l\\l\\le\" would write \"hello\" by writing \"hllo,\"\n"));
_tprintf(_T("moving left three characters, and inserting the \"e\" last.\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Special Keys:\n"));
_tprintf(_T("    \\b      Backspace\n"));
_tprintf(_T("    \\e      Escape\n"));
_tprintf(_T("    \\t      Tab\n"));
_tprintf(_T("    \\n      Enter/Return\n"));
_tprintf(_T("\n"));
_tprintf(_T("For example, \"Username\\tPassword\\n\" would type a username, push tab,\n"));
_tprintf(_T("type a password, and hit enter.\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Named Keys:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\{F1}       F1 key\n"));
_tprintf(_T("    \\{F2}       F2 key\n"));
_tprintf(_T("    ...\n"));
_tprintf(_T("    \\{F24}      F24 key\n"));
_tprintf(_T("    \\{Delete}   Delete\n"));
_tprintf(_T("    \\{End}      End\n"));
_tprintf(_T("    \\{Home}     Home\n"));
_tprintf(_T("    \\{Insert}   Insert\n"));
_tprintf(_T("    \\{PgUp}     PageUp\n"));
_tprintf(_T("    \\{PgDn}     PageDown\n"));
_tprintf(_T("    \\{PrSc}     PrintScreen\n"));
_tprintf(_T("    \\{Pause}    Pause\n"));
_tprintf(_T("    \\{ScLk}     ScrollLock\n"));
_tprintf(_T("\n"));
_tprintf(_T("Keys may be abbreviated when unambiguous; delete = \"\\{d}\" etc.\n"));
_tprintf(_T("For example, \"\\{pd}\\{pd}\\{e}\" would send PageDown twice, then the \"End\" key.\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Numeric Keypad:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\{n0}       Keypad Zero\n"));
_tprintf(_T("    \\{n1}       Keypad One\n"));
_tprintf(_T("    ...\n"));
_tprintf(_T("    \\{n9}       Keypad Nine\n"));
_tprintf(_T("    \\{n*}       Keypad *\n"));
_tprintf(_T("    \\{n+}       Keypad +\n"));
_tprintf(_T("    \\{n-}       Keypad -\n"));
_tprintf(_T("    \\{n/}       Keypad /\n"));
_tprintf(_T("    \\{n.}       Keypad .\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Mouse Commands:\n"));
_tprintf(_T("\n"));
_tprintf(_T("Mouse commands may be given as the special key \"m\" followed by button index,\n"));
_tprintf(_T("event, coordinates, and modifiers.  For example, to click the left\n"));
_tprintf(_T("mouse button at client-local coordinates (200,100), use this syntax:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\{m,l,click,200,100,local}\n"));
_tprintf(_T("\n"));
_tprintf(_T("This could also be abbreviated to \"\\{m,l,c,200,100,l}\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("\"click\" can be replaced with \"down\" or \"up\" if desired.  If the specified\n"));
_tprintf(_T("button index is \"wheel\" (or simply \"w\"), then up/down means to roll the wheel\n"));
_tprintf(_T("up or down, and \"click\" has no meaning (the wheel as a button is usually\n"));
_tprintf(_T("the middle button).  To move the mouse without pressing a button,\n"));
_tprintf(_T("specify \"none\" (or \"n\") as the action.  The buttons may be any of the\n"));
_tprintf(_T("predefined buttons \"l,\" \"m,\" \"r,\" \"1,\" \"2\" (left, middle, right, extra 1,\n"));
_tprintf(_T("extra 2).\n"));
_tprintf(_T("\n"));
_tprintf(_T("The last option can be \"local\" or \"screen\" (\"l\" or \"s\"), and makes the given\n"));
_tprintf(_T("coordinates either local to the client window, or global (to the desktop).\n"));
_tprintf(_T("\n"));
_tprintf(_T("Some more examples:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    Press and hold the right mouse button at screen coordinates (100,50):\n"));
_tprintf(_T("\n"));
_tprintf(_T("       \\{m,r,d,100,50,s}\n"));
_tprintf(_T("\n"));
_tprintf(_T("    Scroll the mouse wheel up at local coordinates (0,0):\n"));
_tprintf(_T("\n"));
_tprintf(_T("       \\{m,w,u,0,0,l}\n"));
_tprintf(_T("\n"));
_tprintf(_T("    Move the mouse to screen coordinates (400,300):\n"));
_tprintf(_T("\n"));
_tprintf(_T("       \\{m,n,n,400,300,s}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("Relative movement:\n"));
_tprintf(_T("       \n"));
_tprintf(_T("    Move the mouse 100 pixels to the right of its current position:\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{m,n,n,100r,0r,s}\n"));
_tprintf(_T("\n"));
_tprintf(_T("    Move the mouse 100 pixels up and click the left button:\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{m,l,c,0r,-100r,s}\n"));
_tprintf(_T("\n"));
_tprintf(_T("Note:  The mouse cursor is always moved to the selected location prior\n"));
_tprintf(_T("       to executing any actions.\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Window Positioning:\n"));
_tprintf(_T("\n"));
_tprintf(_T("The active window's size and position may be changed as shown in the\n"));
_tprintf(_T("following examples:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    Move the active window to absolute position (200,100) and resize\n"));
_tprintf(_T("    it to 640x480:\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{wp,200,100}\\{ws,640,480}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("    Move the active window four pixels to the right and two pixels up\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{wp,4r,-2r}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("    Move the active window to absolute position (1000,1100) using the\n"));
_tprintf(_T("    bottom-right corner of the window as the anchor.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{wp,1000x,-1100x}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("    Resize the current window to 320x240\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{ws,320,240}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("    Expand the current window by 100 pixels horizontally\n"));
_tprintf(_T("    \n"));
_tprintf(_T("       \\{ws,100r,0}\n"));
_tprintf(_T("       \n"));
_tprintf(_T("Note that a zero window size is simply ignored.  A zero position,\n"));
_tprintf(_T("however, indicates the top or left side of the screen.  You may also\n"));
_tprintf(_T("place a position and size command in the same string, if you like\n"));
_tprintf(_T("(this causes only one MoveWindow call to be made, which may or may\n"));
_tprintf(_T("not be important).\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\{wp,200,100,640,480}\n"));
_tprintf(_T("    \n"));
_tprintf(_T("You may use the letters 'h' and 'w' in the numeric portion of the call\n"));
_tprintf(_T("to represent the current height and width, respectively, of the Windows\n"));
_tprintf(_T("desktop.  The following call would move the window to position (1500,800)\n"));
_tprintf(_T("if your desktop resolution is 1600x1200:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\{wp,w-100,h-400}\n"));
_tprintf(_T("    \n"));
_tprintf(_T("If you use the capital versions of 'H' or 'W' then the value used will be\n"));
_tprintf(_T("the \"work area\" dimensions (those of the screen minus the task bar or\n"));
_tprintf(_T("other toolbars.\n"));
_tprintf(_T("\n"));
_tprintf(_T("If you use \"ww\" or \"hh\" then the current window width or height will be used.\n"));
_tprintf(_T("    \n"));
_tprintf(_T("(Do not attempt complex mathematics; what you see is about all you get.)\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Windows Messages:\n"));
_tprintf(_T("\n"));
_tprintf(_T("You may send arbitrary windows messages to the active window during\n"));
_tprintf(_T("playback.  the wParam and lParam may be omitted if '0' is desired for their\n"));
_tprintf(_T("values.  For example:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send notepad \"\\[WM_SYSCOMMAND,SC_MINIMIZE]\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("will minimize the notepad window.  Note that this will take focus away\n"));
_tprintf(_T("from this window and further keystrokes may not be recognized. \n"));
_tprintf(_T("Further windows messages, however, will be sent.  Windows message\n"));
_tprintf(_T("sending is generally for advanced users, but a few common ones are\n"));
_tprintf(_T("listed here for convenience:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\[WM_CLOSE]                     Will close the active window\n"));
_tprintf(_T("    \\[WM_SYSCOMMAND,SC_MINIMIZE]    Minimizes the active window\n"));
_tprintf(_T("    \\[WM_SYSCOMMAND,SC_MAXIMIZE]    Maximizes the active window\n"));
_tprintf(_T("    \\[WM_SYSCOMMAND,SC_RESTORE]     \"Un-minimizes\" the active window\n"));
_tprintf(_T("\n"));
_tprintf(_T("You may specify decimal or hexadecimal numbers (with the \"0x\" prefix), such\n"));
_tprintf(_T("as in the following example:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    \\[WM_COMMAND,1,0x4007]\n"));
_tprintf(_T("    \n"));
_tprintf(_T("String pointers may be specified using single quotes.  For example, this will\n"));
_tprintf(_T("directly set the text in the first child window of notepad (the edit control)\n"));
_tprintf(_T("without using keystrokes.  Using a windows message like this allows you to\n"));
_tprintf(_T("perform window operations without bringing the window to the foreground.\n"));
_tprintf(_T("To insert single quotes, escape them with a backslash:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send -i c notepad \"\\[WM_SETTEXT,0,'Insert this \\'text\\' here']\"\n"));
_tprintf(_T("    \n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Pausing during sending of keystrokes.\n"));
_tprintf(_T("\n"));
_tprintf(_T("You may have the program pause for a user-specified number of milliseconds\n"));
_tprintf(_T("during playback of keystrokes by including \\{sX} in the send string, where\n"));
_tprintf(_T("X is an integer.  For example:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send notepad \"A\\{s500}B\\{s1000}C\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("would send the \"A\" key, wait half a second, send the \"B\" key, wait 1\n"));
_tprintf(_T("second, and then send the \"C\" key.\n"));
_tprintf(_T("\n"));
_tprintf(_T("----------------------------------------------------------------------\n"));
_tprintf(_T("\n"));
_tprintf(_T("Sending messages to dialog IDs.\n"));
_tprintf(_T("\n"));
_tprintf(_T("If you want to send a windows message (or window positioning command)\n"));
_tprintf(_T("to a child window, you can specify a dialog ID or chain of IDs in the\n"));
_tprintf(_T("following manner:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send -i id,id,id,... <windowname>\n"));
_tprintf(_T("\n"));
_tprintf(_T("Where \"id\" can either be a numeric dialog ID (decimal by default, or prefix\n"));
_tprintf(_T("with 0x to use hex) or one of these characters:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    p   previous window\n"));
_tprintf(_T("    n   next window\n"));
_tprintf(_T("    f   first window\n"));
_tprintf(_T("    l   last window\n"));
_tprintf(_T("    c   first child window\n"));
_tprintf(_T("    o   owner window\n"));
_tprintf(_T("\n"));
_tprintf(_T("For example, to move the \"Save As...\" button in the Display Properties\n"));
_tprintf(_T("window to the upper left corner of its parent client area, you could use\n"));
_tprintf(_T("this command:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send -i c,c,n,n,n \"Display Properties\" \\{wp,0,0}\n"));
_tprintf(_T("\n"));
_tprintf(_T("You can specify a dialog item's control ID or window handle with the special\n"));
_tprintf(_T("text 'id' and 'hwnd'.  Also, the delimiter \":\" can be used to represent a\n"));
_tprintf(_T("hiword/loword split.\n"));
_tprintf(_T("\n"));
_tprintf(_T("When sending notifications such as BN_CLICKED, you will most likely want to\n"));
_tprintf(_T("use the -p option to send the actual notification to the parent of the\n"));
_tprintf(_T("control, rather than the control itself.  For example:\n"));
_tprintf(_T("\n"));
_tprintf(_T("    send -pi c,n,n,n,n,n,n \"Run\" \"\\[WM_COMMAND,BN_CLICKED:id,hwnd]\"\n"));
_tprintf(_T("\n"));
_tprintf(_T("With a bit of effort, you can reposition virtually any controls on any\n"));
_tprintf(_T("window, modify text, etc.\n"));
_tprintf(_T("\n"));
}
